@@ -1,35 +1,3 @@
-#--
-# =============================================================================
-# Copyright (c) 2004, Jamis Buck (jgb3@email.byu.edu)
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-# 
-#     * Redistributions of source code must retain the above copyright notice,
-#       this list of conditions and the following disclaimer.
-# 
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-# 
-#     * The names of its contributors may not be used to endorse or promote
-#       products derived from this software without specific prior written
-#       permission.
-# 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
-# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
-# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-# =============================================================================
-#++
-
 require 'sqlite3/constants'
 require 'sqlite3/errors'
 
@@ -65,6 +33,9 @@ module SQLite3
     # to the first row of the result set.
     def commence
       result = @driver.step( @stmt.handle )
+      if result == Constants::ErrorCode::ERROR
+        @driver.reset( @stmt.handle )
+      end
       check result
       @first_row = true
     end
@@ -121,13 +92,16 @@ module SQLite3
       unless @eof
         row = []
         @driver.data_count( @stmt.handle ).times do |column|
-          case @driver.column_type( @stmt.handle, column )
-            when Constants::ColumnType::NULL then
-              row << nil
-            when Constants::ColumnType::BLOB then
-              row << @driver.column_blob( @stmt.handle, column )
-            else
-              row << @driver.column_text( @stmt.handle, column )
+          type	= @driver.column_type( @stmt.handle, column )
+
+          if type == Constants::ColumnType::TEXT
+            row << @driver.column_text( @stmt.handle, column )
+          elsif type == Constants::ColumnType::NULL
+            row << nil
+          elsif type == Constants::ColumnType::BLOB
+            row << @driver.column_blob( @stmt.handle, column )
+          else
+            row << @driver.column_text( @stmt.handle, column )
           end
         end
 
